
# q-expansion of newform 6078.2.a.c, downloaded from the LMFDB on 04 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6078
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 0], [-1, 0], [0, -2], [2, -1], [4, 0], [-4, 0], [-1, 3], [3, 3], [4, -1], [-6, 0], [3, -7], [3, -1], [2, 5], [0, 5], [10, 0], [-7, 7], [6, 3], [-11, 1], [6, -8], [4, -3], [5, -9], [4, -8], [6, -6], [8, -6], [7, -3], [-2, -10], [14, 3], [6, -1], [-11, 1], [12, -10], [-2, 8], [-10, 11], [2, 1], [-12, 0], [-2, -12], [4, 0], [-12, 2], [-8, 8], [9, 1], [5, -13], [-1, 5], [20, -2], [-10, 22], [-16, 1], [12, -17], [-6, 12], [-4, 1], [6, 6], [10, 9], [6, 6], [14, 0], [10, -20], [-24, 5], [4, -10], [26, -6], [28, -4], [8, 12], [-14, 8], [1, -3], [-2, -2], [-8, 3], [-16, 26], [22, 0], [-11, -3], [0, -2], [-6, 8], [20, -7], [18, -12], [-8, 16], [6, -17], [-12, 2], [10, -16], [10, 16], [12, -16], [19, -13], [24, -16], [14, 10], [4, 0], [9, -23], [-14, 28], [12, -2], [-10, 9], [-4, -1], [8, 11], [0, 12], [23, -19], [12, -20], [-10, 12], [-12, 27], [-6, 7], [18, 15], [-13, 11], [24, -2], [-2, -8], [14, -32], [8, -3], [7, -27], [-26, 24], [8, -8], [2, -18], [16, 2], [-20, 35], [9, -25], [0, 24], [-12, -9], [22, 4], [0, -12], [-27, -3], [20, -10], [12, 16], [-10, -23], [24, -24], [2, 12], [-8, 14], [4, 28], [14, -8], [-10, -2], [28, -24], [28, 3], [0, 24], [-5, 3], [-39, 15], [7, 13], [19, 5], [-20, 13], [-28, 15], [6, -10], [0, 20], [32, 4], [-30, 23], [25, 9], [-24, 20], [4, -4], [0, -24], [47, -5], [28, -2], [-24, 7], [24, -14], [6, -28], [6, -14], [24, -12], [8, 26], [-16, -2], [-38, 11], [-2, -25], [26, 10], [-50, 2], [-10, 32], [29, -3], [44, -12], [-34, 32], [-11, 1], [-29, 7], [-20, -1], [-20, -12], [18, -46], [-16, 14], [-18, 14], [29, -13], [26, -2], [-32, -16], [2, 24], [-17, 1], [40, -16], [-22, 30], [-14, 2], [-24, -12], [32, 4], [19, 5], [1, 0], [32, -14], [-18, 35], [16, 2], [-16, -16], [31, 5], [-24, 14], [22, -20], [-22, -8], [33, -1], [12, 20], [-6, 8], [-2, -22], [-42, 33], [-6, -12], [-44, 27], [7, -39], [-2, 12], [-20, 7], [-36, -6], [3, 15], [-18, 4], [-26, 6], [-40, 28], [-16, -24], [-14, -13], [-6, 12], [6, 8], [23, -13], [-2, -32], [-24, -5], [-1, 25], [42, -30], [-4, 2], [-34, 30], [19, -11], [-7, 3], [-20, 34], [3, 17], [47, -9], [-45, 31], [-17, 17], [-58, 10], [11, 5], [-6, 38], [23, -17], [-8, -22], [21, 15], [40, -20], [16, 22], [-11, 7], [-38, -4], [5, -5], [10, 20], [5, 19], [17, -17], [51, -37], [39, -17], [-31, 53], [41, -21], [8, -14], [2, -34], [-16, -32], [20, 32], [-50, 30], [-44, 38], [32, -65], [-36, 29], [30, -20], [0, 20], [20, -52], [-56, -10], [18, -30], [30, 12], [-54, 0], [72, -2], [-38, 38], [38, -40], [-26, -4], [-24, 44], [-16, -12], [-34, 15], [25, -23], [37, -31], [-38, 44], [8, -1], [3, -35], [58, -5], [-13, 43], [22, -6], [-20, -4], [0, 0], [-35, 35], [42, -17], [35, -53], [-52, 12], [44, -33], [-34, 66], [42, -10], [1, 13], [-14, -2], [0, 30], [26, -12], [23, 17], [-46, 51], [0, -20], [-4, 32], [7, 25], [6, -28], [24, -53], [-30, 19], [-19, 37], [18, -14], [-62, 0], [-10, 8], [-18, 4], [-8, 51], [14, -24], [29, 3], [24, 0], [-18, 43], [20, 16], [-38, 6], [12, -38], [72, -20], [-18, 36], [6, -30], [0, 0], [-31, -1], [-45, 33], [16, -46], [36, -38], [-46, 10], [-24, 28], [-62, -6], [39, -21], [-11, 15], [48, -48], [-34, -4], [-43, 29], [-46, 53], [-30, -10], [-16, 23], [-50, -16], [-4, 26], [30, 24], [34, -48], [62, 2], [0, 32], [-75, -1], [-38, 63], [18, 4], [12, -49], [-38, 54], [10, 28], [-22, 28], [-6, -12], [70, -18], [-8, 60], [55, -41], [-46, -18], [4, 34], [-24, 35], [12, -12], [46, -40], [-73, 27], [-38, -16], [-51, 41], [18, -48], [82, -14], [-2, 23], [-34, 61], [0, 34], [-35, 51], [22, -48], [10, -44], [-56, 8], [-18, -42], [60, 9], [36, -13], [-2, 0], [32, -25], [32, -36], [-26, -6], [27, 29], [-30, -21], [-4, 56], [1, 41], [-10, 50], [-10, -16], [-8, 47], [20, -16], [58, -25], [14, 18], [-22, -18], [-28, -41], [10, 4], [88, -14], [36, 12], [-32, 38], [-12, 22], [80, -6], [-26, 16], [20, -37], [-46, 32], [-30, 16], [-16, 32], [-84, 0], [26, 24], [-30, 10], [-4, 44], [-4, 64], [-61, -1], [-24, -32], [-6, 46], [8, -50], [16, 41], [32, -14], [-74, 11], [37, 21], [-16, -4], [40, -7], [42, 20], [-46, 20], [60, -19], [-35, -11], [-2, 40], [38, 3], [61, -35], [0, -48], [26, -10], [47, 11], [22, 23], [-34, 19], [-22, 40], [-8, -33], [-40, 28], [10, -56], [-66, -21], [-10, -2], [50, 3], [-7, -1], [4, -7], [-52, -28], [-22, 18], [-5, -51], [-76, 28], [-31, 17], [22, -57], [-78, -9], [37, -87], [85, 7], [76, 2], [32, 32], [-38, -24], [46, 18], [28, 45], [36, -82], [78, -21], [44, -42], [36, -12]]
